<?php
/* --------------------------------------------------------------
   ParcelServiceFilter.php 2020-04-16
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\ParcelService\SqlCriteria;

use Gambio\Core\Criteria\SqlFilter;
use Webmozart\Assert\Assert;

/**
 * Class ParcelServiceFilter
 *
 * @package Gambio\Admin\ParcelService\SqlCriteria
 */
class ParcelServiceFilter implements SqlFilter
{
    /**
     * @var string
     */
    private $attribute;
    
    /**
     * @var string
     */
    private $value;
    
    /**
     * @var string
     */
    private $operation;
    
    /**
     * @var string[]
     */
    private static $attributeColumnMapping = [
        'id'                   => 'parcel_services.parcel_service_id',
        'name'                 => 'parcel_services.name',
        'isDefault'            => 'parcel_services.default',
        'descriptions.url'     => 'parcel_services_description.url',
        'descriptions.comment' => 'parcel_services_description.comment',
    ];
    
    /**
     * @var string[]
     */
    private static $allowedForLikeOperation = [
        'name',
        'descriptions.url',
        'descriptions.comment',
    ];
    
    /**
     * @var string[]
     */
    private static $allowedForNumericOperation = [
        'id',
    ];
    
    
    /**
     * ParcelServiceFilter constructor.
     *
     * @param string $attribute
     * @param string $value
     * @param string $operation
     */
    public function __construct(string $attribute, string $value, string $operation)
    {
        $this->attribute = $attribute;
        $this->value     = $value;
        $this->operation = $operation;
    }
    
    
    /**
     * @param string $attribute
     * @param string $value
     * @param string $operation
     *
     * @return ParcelServiceFilter
     */
    public static function create(string $attribute, string $value, string $operation = 'like'): ParcelServiceFilter
    {
        $attributes = array_keys(self::$attributeColumnMapping);
        Assert::oneOf($attribute,
                      $attributes,
                      'Provided attribute needs to be one of: ' . implode(', ', $attributes) . '; Got: ' . $attribute);
        
        $operations = ['like', 'eq', 'gt', 'gte', 'lt', 'lte'];
        Assert::oneOf($operation,
                      $operations,
                      'Provided operation needs to be one of: ' . implode(', ', $operations) . '; Got: ' . $operation);
        
        if ($operation === 'like') {
            Assert::oneOf($attribute,
                          self::$allowedForLikeOperation,
                          'Provided attribute is not allowed for "like" operation.');
        } elseif (in_array($operation, ['gt', 'gte', 'lt', 'lte'])) {
            Assert::oneOf($attribute,
                          self::$allowedForNumericOperation,
                          'Provided attribute is not allowed for a numeric operation.');
        }
        
        return new self($attribute, $value, $operation);
    }
    
    
    /**
     * @return string
     */
    public function attribute(): string
    {
        return $this->attribute;
    }
    
    
    /**
     * @return string
     */
    public function value(): string
    {
        return $this->value;
    }
    
    
    /**
     * @return string
     */
    public function operation(): string
    {
        return $this->operation;
    }
    
    
    /**
     * @return string
     */
    public function column(): string
    {
        return self::$attributeColumnMapping[$this->attribute];
    }
}